@@ -1,4 +1,4 @@
-// Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License"). You may
 // not use this file except in compliance with the License. A copy of the
@@ -60,6 +60,26 @@ func TestExtractRegistry(t *testing.T) {
 			Service: ServiceECR,
 		},
 		hasError: false,
+	}, {
+		// Dual-stack endpoint
+		serverURL: "123456789012.dkr-ecr.us-west-2.on.aws",
+		registry: &Registry{
+			ID:      "123456789012",
+			FIPS:    false,
+			Region:  "us-west-2",
+			Service: ServiceECR,
+		},
+		hasError: false,
+	}, {
+		// Dual-stack FIPS endpoint
+		serverURL: "123456789012.dkr-ecr-fips.us-west-2.on.aws",
+		registry: &Registry{
+			ID:      "123456789012",
+			FIPS:    true,
+			Region:  "us-west-2",
+			Service: ServiceECR,
+		},
+		hasError: false,
 	}, {
 		serverURL: "210987654321.dkr.ecr.cn-north-1.amazonaws.com.cn/foo",
 		registry: &Registry{
@@ -69,6 +89,62 @@ func TestExtractRegistry(t *testing.T) {
 			Service: ServiceECR,
 		},
 		hasError: false,
+	}, {
+		// IPv6 CN
+		serverURL: "210987654321.dkr.ecr.cn-north-1.on.amazonwebservices.com.cn",
+		registry: &Registry{
+			ID:      "210987654321",
+			FIPS:    false,
+			Region:  "cn-north-1",
+			Service: ServiceECR,
+		},
+		hasError: false,
+	}, {
+		serverURL: "210987654321.dkr.ecr.us-iso-east-1.c2s.ic.gov",
+		registry: &Registry{
+			ID:      "210987654321",
+			FIPS:    false,
+			Region:  "us-iso-east-1",
+			Service: ServiceECR,
+		},
+		hasError: false,
+	}, {
+		serverURL: "123456789012.dkr.ecr.us-isob-east-1.sc2s.sgov.gov",
+		registry: &Registry{
+			ID:      "123456789012",
+			FIPS:    false,
+			Region:  "us-isob-east-1",
+			Service: ServiceECR,
+		},
+		hasError: false,
+	}, {
+		serverURL: "123456789012.dkr.ecr.eu-isoe-west-1.cloud.adc-e.uk",
+		registry: &Registry{
+			ID:      "123456789012",
+			FIPS:    false,
+			Region:  "eu-isoe-west-1",
+			Service: ServiceECR,
+		},
+		hasError: false,
+	}, {
+		serverURL: "123456789012.dkr.ecr.us-isof-east-1.csp.hci.ic.gov",
+		registry: &Registry{
+			ID:      "123456789012",
+			FIPS:    false,
+			Region:  "us-isof-east-1",
+			Service: ServiceECR,
+		},
+		hasError: false,
+	}, {
+		// IPv6 GovCloud
+		serverURL: "123456789012.dkr-ecr.us-gov-east-1.on.aws",
+		registry: &Registry{
+			ID:      "123456789012",
+			FIPS:    false,
+			Region:  "us-gov-east-1",
+			Service: ServiceECR,
+		},
+		hasError: false,
 	}, {
 		serverURL: "123456789012.dkr.ecr-fips.us-gov-west-1.amazonaws.com",
 		registry: &Registry{
@@ -78,6 +154,16 @@ func TestExtractRegistry(t *testing.T) {
 			Service: ServiceECR,
 		},
 		hasError: false,
+	}, {
+		// IPv6 GovCloud FIPS
+		serverURL: "123456789012.dkr-ecr-fips.us-gov-east-1.on.aws",
+		registry: &Registry{
+			ID:      "123456789012",
+			FIPS:    true,
+			Region:  "us-gov-east-1",
+			Service: ServiceECR,
+		},
+		hasError: false,
 	}, {
 		serverURL: "https://public.ecr.aws",
 		registry: &Registry{
@@ -111,6 +197,10 @@ func TestExtractRegistry(t *testing.T) {
 	}, {
 		serverURL: "210987654321.dkr.ecr.cn-north-1.amazonaws.com.cn.fake.example.com.cn",
 		hasError:  true,
+	}, {
+		// China region IPv6 endpoints are "on.amazonwebservices.com.cn"
+		serverURL: "210987654321.dkr.ecr.cn-north-1.on.aws.com.cn",
+		hasError:  true,
 	}, {
 		serverURL: "https://public.ecr.aws.fake.example.com",
 		hasError:  true,
